// Datei : DX8Sound.cpp

// --------------------------------------------------------------------------------------
//
// SoundManager
// zum Abspielen von Wave-Dateien und Musikstcken mit Hilfe der FMOD Library
//
//
// (c) 2002 Jrg M. Winterstein
//
// --------------------------------------------------------------------------------------

// --------------------------------------------------------------------------------------
// Include Dateien
// --------------------------------------------------------------------------------------

#include <stdio.h>
#include "DX8Sound.h"
#include "Logdatei.h"
#include "Globals.h"
#include "Gameplay.h"
#include "Main.h"
#include "Player.h"
#include "Timer.h"
#include "unrarlib.h"

//---------------------------------------------------------------------------------------
// Funktionsdefinitionen
//---------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Name			: GetFMODErrorString
// Aufgabe		: FMOD Error-Wert in String umwandeln
// Rckgabewert	: String mit Error-Text
// Parameter	: FMOD Error-Wert
//-----------------------------------------------------------------------------

char *GetFMODErrorString(int ErrorNr)
{
	char ErrorString[100];
	char *ReturnString;

	strcpy_s(ErrorString, strlen("Unknown Error") + 1, "Unknown Error");

	/*switch(ErrorNr)
	{
		case FMOD_ERR_NONE				: strcpy_s(ErrorString, "No errors"); break;
		case FMOD_ERR_BUSY				: strcpy_s(ErrorString, "Cannot call this command after FSOUND_Init. Call FSOUND_Close first. "); break;
		case FMOD_ERR_UNINITIALIZED		: strcpy_s(ErrorString, "This command failed because FSOUND_Init or FSOUND_SetOutput was not called "); break;
		case FMOD_ERR_INIT				: strcpy_s(ErrorString, "Error initializing output device. "); break;
		case FMOD_ERR_ALLOCATED			: strcpy_s(ErrorString, "Error initializing output device, but more specifically, the output device is already in use and cannot be reused. "); break;
		case FMOD_ERR_PLAY				: strcpy_s(ErrorString, "Playing the sound failed. "); break;
 		case FMOD_ERR_OUTPUT_FORMAT		: strcpy_s(ErrorString, "Soundcard does not support the features needed for this soundsystem (16bit stereo output) "); break;
 		case FMOD_ERR_COOPERATIVELEVEL	: strcpy_s(ErrorString, "Error setting cooperative level for hardware. "); break;
 		case FMOD_ERR_CREATEBUFFER		: strcpy_s(ErrorString, "Error creating hardware sound buffer. "); break;
 		case FMOD_ERR_FILE_NOTFOUND		: strcpy_s(ErrorString, "File not found "); break;
 		case FMOD_ERR_FILE_FORMAT		: strcpy_s(ErrorString, "Unknown file format "); break;
 		case FMOD_ERR_FILE_BAD			: strcpy_s(ErrorString, "Error loading file "); break;
 		case FMOD_ERR_MEMORY			: strcpy_s(ErrorString, "Not enough memory or resources "); break;
 		case FMOD_ERR_VERSION			: strcpy_s(ErrorString, "The version number of this file format is not supported "); break;
 		case FMOD_ERR_INVALID_PARAM		: strcpy_s(ErrorString, "An invalid parameter was passed to this function "); break;
 		case FMOD_ERR_NO_EAX			: strcpy_s(ErrorString, "Tried to use an EAX command on a non EAX enabled channel or output. "); break;
 		case FMOD_ERR_CHANNEL_ALLOC		: strcpy_s(ErrorString, "Failed to allocate a new channel "); break;
 		case FMOD_ERR_RECORD			: strcpy_s(ErrorString, "Recording is not supported on this machine "); break;
 		case FMOD_ERR_MEDIAPLAYER		: strcpy_s(ErrorString, "Windows Media Player not installed so cannot play wma or use internet : streaming. "); break;
		default							: strcpy_s(ErrorString, "Unknown Error"); break;
	} // switch(ErrorNr)*/

#if defined(PLATFORM_DIRECTX)
	ReturnString = ErrorString;
#elif defined(PLATFORM_SDL)
    ReturnString = Mix_GetError();
#endif

	return ReturnString;
} // GetFMODErrorString

//---------------------------------------------------------------------------------------
// Klassen-Definition (CSong)
//---------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------
// Name			: CSong (Konstruktor)
// Aufgabe		: Song initialisieren
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

CSong::CSong()
{
	SongData = NULL;
} // Konstruktor

//---------------------------------------------------------------------------------------
// Name			: CSong (Destruktor)
// Aufgabe		: Song initialisieren
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

CSong::~CSong()
{
	// Freigeben
	if (SongData != NULL)
		MUSIC_FreeSong(SongData);

	SongData = NULL;
} // Destruktor

//---------------------------------------------------------------------------------------
// Name			: Update
// Aufgabe		: Songs ggf faden
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

bool CSong::Update(void)
{
	// Spielt der Song garnicht ? Dann brauchen wir ihn auch nicht zu bearbeiten =)
	//
	if (!MUSIC_IsPlaying(SongData))
		return false;

	// Fadet der Song grade ?
	//
	if (FadingVolume != 0.0f)
	{
		Volume += FadingVolume SYNC;

		// Grenze berschritten ?
		if ((FadingVolume > 0.0f &&
		 	 Volume >= FadingEnd) ||

			(FadingVolume < 0.0f &&
			 Volume <= FadingEnd))
		{
			// Beim Anhalten des Songs ausschalten oder pausieren ?
			if (FadingVolume < 0.0f)
			{
				if (FadingPaused == false)
					MUSIC_StopSong(SongData);			// Ganz anhalten ?
				else
					MUSIC_SetPaused(SongData, true);	// oder nur pausieren ?
			}

			FadingVolume = 0.0f;
			Volume		 = float(FadingEnd);
		}
		MUSIC_SetMasterVolume(SongData, (int)(Volume*pSoundManager->its_GlobalMusicVolume/100.0f*2.55f));
	}

	return true;
} // Update

//---------------------------------------------------------------------------------------
// Klassen-Definition (CWave)
//---------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------
// Name			: CWave (Konstruktor)
// Aufgabe		: Sound initialisieren
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

CWave::CWave()
{
	isPlaying = false;
	isLooped  = false;
	FadeMode = FADEMODE_NON;
	Channel   = -1;
	SoundData = NULL;
} // Konstruktor

//---------------------------------------------------------------------------------------
// Name			: ~CWave (Destruktor)
// Aufgabe		: Sound lschen
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

CWave::~CWave()
{
	// Freigeben
	if (SoundData != NULL)
		SOUND_Sample_Free(SoundData);

	SoundData = NULL;
} // Destruktor

//---------------------------------------------------------------------------------------
// Name			: Update
// Aufgabe		: Channel prfen
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

bool CWave::Update()
{
	if (SOUND_IsPlaying(Channel) == false)
	{
		isPlaying = false;
		FadeMode = FADEMODE_NON;
	}

	float vol = (float)SOUND_GetVolume(Channel);

	// fading?
	//
	switch (FadeMode)
	{
		// momentan nicht faden
		//
		case FADEMODE_NON:
		break;

		// einfaden
		//
		case FADEMODE_IN:
		{
			// Volume erhhen
			//
			vol += 30.0f SYNC;

			// maximum? Dann fading anhalten
			//
			if (vol > pSoundManager->its_GlobalSoundVolume * 2.55f)
			{
				vol = pSoundManager->its_GlobalSoundVolume * 2.55f;
				FadeMode = FADEMODE_FULL;
			}

			// Lautstrke setzen
			//
			SOUND_SetVolume(Channel, (int)vol);
		}
		break;

		// ausfaden
		//
		case FADEMODE_OUT:
		{
			// Volume verringern
			//
			vol -= 30.0f SYNC;

			// < 0 ? Dann anhalten
			//
			if (vol < 0.0f)
			{
				SOUND_StopSound(Channel);
				isPlaying = false;
				Channel = -1;
				FadeMode = FADEMODE_NON;
			}
			else
				SOUND_SetVolume(Channel, (int)vol);
		}
		break;

		default:
			break;
	}

	return true;
} // Update

//---------------------------------------------------------------------------------------
// Klassen-Definition (SoundManager)
//---------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------
// Name			: CSoundManager (Konstruktor)
// Aufgabe		: Standardmssige Initialisierung
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

CSoundManager::CSoundManager()
{
	SOUNDMANAGER_PARAMETERS smpp;

	InitSuccessfull = false;

	its_LoadedSongs  = 0;
	its_LoadedSounds = 0;
	its_GlobalMusicVolume		= 100;
	its_GlobalSoundVolume		= 100;

	smpp.MaxSoftwareChannels	= 64;
	smpp.Mixrate				= 44100;
	smpp.Flags					= FSOUND_INIT_USEDEFAULTMIDISYNTH;

	// Sounds auf NULL setzen
	for (int i=0; i<MAX_SOUNDS; i++)
		its_Sounds[i] = NULL;

	// Songs auf NULL setzen
	for (int i=0; i<MAX_SONGS; i++)
		its_Songs[i] = NULL;

	InitSuccessfull = InitFMOD(smpp);

	if (true == InitSuccessfull)
		Protokoll.WriteText("Sound Manager created\n", false);
	else
		Protokoll.WriteText("\n-> Create Sound Manager failed !\n", false);
}; // Konstruktor

//---------------------------------------------------------------------------------------
// Name			: CSoundManager (berladener Konstruktor)
// Aufgabe		: Initialisierung mit bergebenen Parametern
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

CSoundManager::CSoundManager   (SOUNDMANAGER_PARAMETERS smpp)
{
	InitSuccessfull = false;

	its_LoadedSongs  = 0;
	its_LoadedSounds = 0;
	its_GlobalMusicVolume	= float(smpp.GlobalMusicVolume);
	its_GlobalSoundVolume	= float(smpp.GlobalSoundVolume);

	// Sounds auf NULL setzen
	for (int i=0; i<MAX_SOUNDS; i++)
		its_Sounds[i] = NULL;

	// Songs auf NULL setzen
	for (int i=0; i<MAX_SONGS; i++)
		its_Songs[i] = NULL;

	if (InitFMOD(smpp) == true)
	{
		Protokoll.WriteText("Sound Manager created\n", false);
	}
	else
	{
		Protokoll.WriteText("\n-> Create Sound Manager failed !\n", true);
	}
}; // berladener Konstruktor


//---------------------------------------------------------------------------------------
// Name			: ~CSoundManager (Destruktor)
// Aufgabe		: Songs freigeben
// Rckgabewert	: Keiner
// Parameter	: Keine
//---------------------------------------------------------------------------------------

CSoundManager::~CSoundManager()
{
	Protokoll.WriteText("-> Shutting down Sound Manager\n", false);

	MUSIC_StopAllSongs();

	// Songs freigeben
	//
	for (int i=0; i<MAX_SONGS; i++)
		if (its_Songs[i] != NULL)
		{
			delete(its_Songs[i]);	// Freigeben
			its_Songs[i] = NULL;
		}

	its_LoadedSongs = 0;

	// Sounds freigeben
	for (int i=0; i<MAX_SOUNDS; i++)
		if (its_Sounds[i] != NULL)
		{
			delete(its_Sounds[i]);	// Freigeben
			its_Sounds[i] = NULL;
		}

	its_LoadedSounds = 0;

	// FSOUND beenden
	//
	SOUND_Close();

}; // Desktruktor

//---------------------------------------------------------------------------------------
// Name			: InitFMOD
// Aufgabe		: FMOD Initialisieren
// Rckgabewert	: true bei Erfolg, false bei nicht-Erfolg
// Parameter	: keine
//---------------------------------------------------------------------------------------

bool CSoundManager::InitFMOD(SOUNDMANAGER_PARAMETERS smpp)
{
#if defined(PLATFORM_DIRECTX)
	Protokoll.WriteText ("Initializing FMOD\n", false);

	FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND);					// Output-Mode setzen
	FSOUND_SetDriver(0);									// Default-Soundkarte setzen
	FSOUND_SetMixer(FSOUND_MIXER_QUALITY_AUTODETECT);		// Mixer-Quality setzen
#elif defined(PLATFORM_SDL)
	Protokoll.WriteText ("Initializing SDL_mixer\n", false);
#endif

	InitSuccessfull = (TRUE == SOUND_Init(smpp.Mixrate, smpp.MaxSoftwareChannels, smpp.Flags));

	if (false == InitSuccessfull)
	{
		Protokoll.WriteText ("\n->", false);
		Protokoll.WriteText (GetFMODErrorString(SOUND_GetError()), false);
		return false;
	}

	MaxChannels = SOUND_GetMaxChannels();
	Protokoll.WriteText ("Successfull\n", false);

	return true;
} // InitFMOD

//---------------------------------------------------------------------------------------
// Name			: SetVolumes
// Aufgabe		: neue globale Volumes setzen
// Rckgabewert	: keiner
// Parameter	: Sound		neue Sound-Lautstrke
//				  Musik		neue Musik-Lautstrke
//---------------------------------------------------------------------------------------

void CSoundManager::SetVolumes(float Sound, float Musik)
{
	its_GlobalSoundVolume = Sound;
	its_GlobalMusicVolume = Musik;
} // SetVolumes

//---------------------------------------------------------------------------------------
// Name			: LoadSong
// Aufgabe		: Song laden
// Rckgabewert	: Liefert Integer Wert der Arraypos des Songs zurck, -1 bei Fehler
// Parameter	: Filename	Name des zu ladenden Songs
//---------------------------------------------------------------------------------------

bool CSoundManager::LoadSong(const char *Filename, int Nr)
{
	if (false == InitSuccessfull)
    {
    	/* Need this even if sound output not possible, because lots of code
           accesses its_Songs[] */

	    if (its_Songs[Nr] != NULL)
	    {
		    delete(its_Songs[Nr]);
	    }
	    its_Songs[Nr] = new CSong();
        
		return false;
    }

	bool			fromrar;
	char			*pData;
	char			Temp[256];
	unsigned long	Size;
	char Buffer[100];

	fromrar = false;

	// Zuerst checken, ob sich der Song in einem MOD-Ordner befindet
	if (CommandLineParams.RunOwnLevelList == true)
	{
		sprintf_s(Temp, "levels/%s/%s", CommandLineParams.OwnLevelList, Filename);
		if (FileExists(Temp))
			goto loadfile;
	}

	// Dann checken, ob sich das File im Standard Ordner befindet
	sprintf_s(Temp, "data/%s", Filename);
	if (FileExists(Temp))
		goto loadfile;

	// Auch nicht? Dann ist es hoffentlich im RAR file
	if (urarlib_get(&pData, &Size, Filename, RARFILENAME, convertText(RARFILEPASSWORD)) == false)
	{
		sprintf_s(Temp, "\n-> Error loading %s from Archive - Dx8Sound.cpp 1 !\n", Filename);
		Protokoll.WriteText(Temp, true);
		return false;
	}
	else
		fromrar = true;

loadfile:

	int Nummer = Nr;
	if (its_Songs[Nummer] != NULL)
	{
		delete(its_Songs[Nummer]);
		its_Songs[Nummer] = NULL;
	}

	its_Songs[Nummer] = new CSong();

	if (fromrar == false)
		its_Songs[Nummer]->SongData = MUSIC_LoadSong(Temp);
	else
	{
		its_Songs[Nummer]->SongData = MUSIC_LoadSongEx(pData, 0, Size, FSOUND_LOADMEMORY, NULL, 0);
		free(pData);
	}

	// Fehler beim Laden ?
	//
	if (its_Songs[Nummer]->SongData == NULL)
	{
		strcpy_s(Buffer, strlen("\n-> Error loading ") + 1, "\n-> Error loading ");
		strcat_s(Buffer, strlen(Filename) + 1, Filename);
		strcat_s(Buffer, 3, "\n");
		Protokoll.WriteText(Buffer, true);

		//return (-1);
		return false;
	}

	its_LoadedSongs++;

	//its_Songs[Nummer].Name = Filename;
	its_Songs[Nummer]->Volume = 100;
	its_Songs[Nummer]->FadingVolume = 0.0f;

	// TODO FIX
	/*
	strcpy_s(Buffer, "Load Song : ");
	strcat_s(Buffer, Filename);
	strcat_s(Buffer, " successfull\n");
	Protokoll.WriteText(Buffer, false);
	*/

	//return Nummer;
	return true;
} // LoadSong

//---------------------------------------------------------------------------------------
// Name			: PlaySong
// Aufgabe		: Song abspielen
// Rckgabewert	: keiner
// Parameter	: Nr		Array-Nummer des Songs
//				  Paused	Neu starten oder Pause aufheben
//---------------------------------------------------------------------------------------

bool CSoundManager::PlaySong(int Nr, bool Paused)
{
	if (false == InitSuccessfull)
		return false;

	if (its_Songs[Nr] == NULL)
		return false;

	if (Paused == false)
	{
		MUSIC_PlaySong(its_Songs[Nr]->SongData);			// Von vorne abspielen ?
		its_Songs[Nr]->FadingVolume = 0.0f;
		SetSongVolume(Nr, its_GlobalMusicVolume);
	}
	else
	{
		SetSongVolume(Nr, its_Songs[Nr]->Volume);
		MUSIC_SetPaused(its_Songs[Nr]->SongData, false);	// oder Pause aufheben ?
	}

	CurrentSongNr = Nr;

	return true;
} // PlaySong

//---------------------------------------------------------------------------------------
// Name			: StopSong
// Aufgabe		: Song anhalten
// Rckgabewert	: false, wenn der Song nicht exisitert, sonst true
// Parameter	: Nr		Array-Nummer des Songs
//				  Paused	Ganz anhalten oder pausieren
//---------------------------------------------------------------------------------------

bool CSoundManager::StopSong(int Nr, bool Paused)
{
	if (its_Songs[Nr] == NULL)
		return false;

	SetSongVolume(Nr, its_Songs[Nr]->Volume);

	if (Paused == false)
		MUSIC_StopSong(its_Songs[Nr]->SongData);			// Ganz anhalten ?
	else
		MUSIC_SetPaused(its_Songs[Nr]->SongData, true);		// oder nur pausieren ?

	return true;
} // StopSong

//---------------------------------------------------------------------------------------
// Name			: StopAllSongs
// Aufgabe		: Alle gespielten Songs werden angehalten
// Rckgabewert	: keiner
//---------------------------------------------------------------------------------------

void CSoundManager::StopAllSongs(bool Paused)
{
	for (int i = 0; i < MAX_SONGS; i++)
		if (its_Songs[i] != NULL &&
			MUSIC_IsPlaying(its_Songs[i]->SongData))
			StopSong(i, Paused);
}

//---------------------------------------------------------------------------------------
// Name			: StopAllLoopedSounds
// Aufgabe		: Alle geloopten Sounds stoppen
// Rckgabewert	: keiner
//---------------------------------------------------------------------------------------

void CSoundManager::StopAllLoopedSounds(void)
{
	for (int i = 0; i < MAX_SOUNDS; i++)
		if (its_Sounds[i] != NULL &&
			its_Sounds[i]->isLooped == true)
			StopWave(i);
}

//---------------------------------------------------------------------------------------
// Name			: StopAllLoopedSounds
// Aufgabe		: Alle geloopten Sounds stoppen
// Rckgabewert	: keiner
//---------------------------------------------------------------------------------------

void CSoundManager::StopAllSounds(void)
{
	for (int i = 0; i < MAX_SOUNDS; i++)
		if (its_Sounds[i] != NULL)
			StopWave(i);
}

//---------------------------------------------------------------------------------------
// Name			: SetSongVolume
// Aufgabe		: Lautstrke eines Songs setzen
// Rckgabewert	: keiner
// Parameter	: Nr		Array-Nummer des Songs
//				  Volume	Zu setzende Lautstrke
//---------------------------------------------------------------------------------------

void CSoundManager::SetSongVolume(int Nr, float Volume)
{
	if (false == InitSuccessfull)
		return;

	its_Songs[Nr]->Volume = Volume;
	MUSIC_SetMasterVolume(its_Songs[Nr]->SongData, (int)(Volume*its_GlobalMusicVolume/100.0f*2.55f));
} // SetSongVolume

//---------------------------------------------------------------------------------------
// Name			: SetAbsoluteSongVolume
// Aufgabe		: aboslute Lautstrke eines Songs setzen
// Rckgabewert	: keiner
// Parameter	: Nr		Array-Nummer des Songs
//				  Volume	Zu setzende Lautstrke
//---------------------------------------------------------------------------------------

void CSoundManager::SetAbsoluteSongVolume(int Nr, float Volume)
{
	if (false == InitSuccessfull)
		return;

	its_Songs[Nr]->Volume = Volume;
	MUSIC_SetMasterVolume(its_Songs[Nr]->SongData, (int)(Volume));
} // SetSongVolume

//---------------------------------------------------------------------------------------
// Name			: SetAllSongVolumes
// Aufgabe		: Lautstrke aller Songs setzen
// Rckgabewert	: keiner
// Parameter	: keine
//---------------------------------------------------------------------------------------

void CSoundManager::SetAllSongVolumes(void)
{
	for (int i=0; i<MAX_SONGS; i++)
		if (its_Songs[i] != NULL)
			SetSongVolume(i, its_GlobalMusicVolume);
} // SetAllSongVolumes

//---------------------------------------------------------------------------------------
// Name			: Update
// Aufgabe		: Songs bei Bedarf faden und benutzte Channels zhlen usw
// Rckgabewert	: keiner
// Parameter	: keine
//---------------------------------------------------------------------------------------

void CSoundManager::Update(void)
{
	if (false == InitSuccessfull)
		return;

	// Sounds durchgehen und updaten
	//
	for (int i=0; i<MAX_SOUNDS; i++)
		if (its_Sounds[i] != NULL)
			its_Sounds[i]->Update();

	// Songs durchgehen und bei Bedarf faden
	//
	for (int i=0; i<MAX_SONGS; i++)
		if (its_Songs[i] != NULL)
			its_Songs[i]->Update();

	// Anzahl benutzter Channels zhlen
	//
	ChannelsUsed = 0;

	for (int i=0; i < MaxChannels; i++)
		if (SOUND_IsPlaying(i))
			ChannelsUsed++;

} // Update

//---------------------------------------------------------------------------------------
// Name			: FadeSong
// Aufgabe		: Fadet einen Song ein/aus und hlt danach an bzw pausiert
// Rckgabewert	: keiner
// Parameter	: Nr			Nummer des Songs
//				  Speed			Geschwindigkeit und Richtung des Fades
//				  End			Grenze des Fadens
//---------------------------------------------------------------------------------------

void CSoundManager::FadeSong(int Nr, float Speed, int End, bool Paused)
{
	if (false == InitSuccessfull)
		return;

	its_Songs[Nr]->FadingVolume	= Speed;
	its_Songs[Nr]->FadingEnd	= (int)(its_GlobalMusicVolume * End / 100.0f);
	its_Songs[Nr]->FadingPaused = Paused;

	// Einfaden ? Dann ggf den Song erst mal starten, je nach Angabe
	// neu von Anfang an oder Pausierung aufheben
	if (Speed > 0.0f)
		PlaySong(Nr, Paused);

} // FadeSong

//---------------------------------------------------------------------------------------
// Name			: FadeWave
// Aufgabe		: Wave ein/ausfadem
// Rckgabewert	: keiner
// Parameter	: Nr			Nummer des Waves
//				  Mode			FadeMode
//---------------------------------------------------------------------------------------

void CSoundManager::FadeWave(int Nr, int Mode)
{
	// Sound soll eingefadet werden?
	if (Mode == FADEMODE_IN)
	{
		// Fadet gerade garnicht? Dann abspielen
		if (its_Sounds[Nr]->FadeMode == FADEMODE_NON)
			PlayWave(1, 128, 11025, Nr);
	}

	// Sound soll ausgefadet werden?
	else
	if (Mode == FADEMODE_OUT)
	{
	}

	its_Sounds[Nr]->FadeMode = Mode;

} // FadeWave

//---------------------------------------------------------------------------------------
// Name			: LoadWave
// Aufgabe		: Ldt eine Wave Datei
// Rckgabewert	: Nr der Wavedatei im Array
// Parameter	: Filename		Dateiname der Wave
//---------------------------------------------------------------------------------------

bool CSoundManager::LoadWave(const char *Filename, int Nr, bool looped)
{
	if(GameRunning == false)
		return false;

	bool			fromrar;
	char			*pData;
	char			Temp[256];
	unsigned long	Size;
	char Buffer[100];

	fromrar = false;

	int Nummer = Nr;
	if (its_Sounds[Nummer] != NULL)
	{
		delete(its_Sounds[Nummer]);
		its_Sounds[Nummer] = NULL;
	}

	if (false == InitSuccessfull)
    {
    	/* Need this even if sound output not possible, because lots of code
           accesses its_Sounds[] */
		its_Sounds[Nummer] = new CWave();
    	
		return false;
	}
    
	// Zuerst checken, ob sich der Sound in einem MOD-Ordner befindet
	if (CommandLineParams.RunOwnLevelList == true)
	{
		sprintf_s(Temp, "levels/%s/%s", CommandLineParams.OwnLevelList, Filename);
		if (FileExists(Temp))
			goto loadfile;
	}

	// Dann checken, ob sich das File im Standard Ordner befindet
	sprintf_s(Temp, "data/%s", Filename);
	if (FileExists(Temp))
		goto loadfile;

	// Auch nicht? Dann ist es hoffentlich im RAR file
	if (urarlib_get(&pData, &Size, Filename, RARFILENAME, convertText(RARFILEPASSWORD)) == false)
	{
		sprintf_s(Temp, "\n-> Error loading %s from Archive - Dx8Sound.cpp 2 !\n", Filename);
		Protokoll.WriteText(Temp, true);
		return false;
	}
	else
		fromrar = true;

loadfile:

	its_Sounds[Nummer] = new CWave();

	// von Platte laden
	if (fromrar == false)
	{
		if (looped == false)
			its_Sounds[Nummer]->SoundData = SOUND_Sample_Load(Nr, Temp, FSOUND_LOOP_OFF, 0, Size);
		else // mit Loop laden
			its_Sounds[Nummer]->SoundData = SOUND_Sample_Load(Nr, Temp, FSOUND_LOOP_NORMAL, 0, Size);
	}
	// aus Rarfile laden
	else
	{
		if (looped == false)
			its_Sounds[Nummer]->SoundData = SOUND_Sample_Load(Nr, pData, FSOUND_LOADMEMORY | FSOUND_LOOP_OFF, 0, Size);
		else // mit Loop laden
			its_Sounds[Nummer]->SoundData = SOUND_Sample_Load(Nr, pData, FSOUND_LOADMEMORY | FSOUND_LOOP_NORMAL, 0, Size);

		free(pData);
	}

	// Temp Datei wieder lschen und Speicher freigeben
	//DeleteFile("temp.dat");
	//free(pData);

	// Und bei bedarf den loop ausschalten
	//if (looped == false)
	//	FSOUND_Sample_SetLoopMode(its_Sounds[Nummer]->SoundData, FSOUND_LOOP_OFF);

	// Fehler beim Laden ?
	//
	if (its_Sounds[Nummer]->SoundData == NULL)
	{
		// TODO FIX
/*
		strcpy_s(Buffer, "\n-> Error loading ");
		strcat_s(Buffer, Filename);
		strcat_s(Buffer, "\n");
		Protokoll.WriteText(Buffer, true);
*/

	//	return (-1);
		return false;
	}

	its_Sounds[Nummer]->isLooped = looped;

	its_LoadedSounds++;

	//its_Sounds[Nummer].Name = Filename;

	// kurz anspielen und wieder stoppen
	int Channel = SOUND_PlaySound   (FSOUND_FREE, its_Sounds[Nr]->SoundData);
	SOUND_SetVolume(Channel, 0);
	SOUND_StopSound(Channel);

	// TODO FIX
/*
		strcpy_s(Buffer, TextArray [TEXT_LADE_WAVE]);
		strcat_s(Buffer, Filename);
		strcat_s(Buffer, TextArray [TEXT_LADEN_ERFOLGREICH]);
		strcat_s(Buffer, " \n");
		Protokoll.WriteText(Buffer, false);*/


	DisplayLoadInfo(Buffer);

	//return Nummer;
	return true;
} // LoadWave

//---------------------------------------------------------------------------------------
// Name			: PlayWave
// Aufgabe		: Spielt eine Wave Datei
// Rckgabewert	: true, wenn der Sound gespielt wurde, sonst false
// Parameter	: Nr des Sounds im Array
//---------------------------------------------------------------------------------------

bool CSoundManager::PlayWave(int Vol, int Pan, int Freq, int Nr)
{
	if (false == InitSuccessfull)
		return false;

	int Channel;		// Benutzter Channel

	// hrt man den Sound berhaupt ?
	//
	if(Vol == 0 ||
		its_GlobalSoundVolume == 0)
		return false;

	if(its_Sounds[Nr] == NULL)
		return false;

	// Sound spielen
	//
	Channel = SOUND_PlaySound   (FSOUND_FREE, its_Sounds[Nr]->SoundData);

	// Kein freier Channel gefunden ?
	//
	if (Channel == -1)
		return false;

	its_Sounds[Nr]->Channel   = Channel;
	its_Sounds[Nr]->isPlaying = true;;

	// Und Werte fr den Channel, in dem er gespielt wird, setzen
	//
	SOUND_SetFrequency(Channel, Freq);
	SOUND_SetVolume   (Channel, int(its_GlobalSoundVolume*Vol/100.0f*2.55f));
	SOUND_SetPan	  (Channel, Pan);

	return true;
} // PlayWave

//---------------------------------------------------------------------------------------
// Name			: PlayWave3D
// Aufgabe		: Spielt eine Wave Datei, je nach Spieler Abstand lauter oder leiser
//				  und setzt das panning entsprechend der Position der Geruschquelle
// Rckgabewert	: true, wenn der Sound gespielt wurde, sonst false
// Parameter	: Nr des Sounds im Array
//---------------------------------------------------------------------------------------

bool CSoundManager::PlayWave3D(int x, int y, int Freq, int Nr)
{
	int   vol, pan;
	float xdiff, ydiff, Abstand;

	xdiff = ((pPlayer[0]->xpos + 45)  - x);
	ydiff = ((pPlayer[0]->ypos + 45)  - y);

	Abstand = float(sqrt((xdiff * xdiff) + (ydiff * ydiff)));

	vol = int(100-float(Abstand/6.0f));
	if (vol < 0)
		vol = 0;
	else
	{
		// Sound links oder rechts vom Spieler ?
		if (x < pPlayer[0]->xpos + 45)
		{
			pan = 128 - (100 - vol);
			if (pan < 0)
				pan = 0;
		}
		else
		{
			pan = 128 + (100 - vol);
			if (pan > 255)
				pan = 255;
		}

		PlayWave(vol, pan, Freq, Nr);
	}

	return true;
} // PlaySound

// --------------------------------------------------------------------------------------
// Lautstrke eines 3D sounds ndern
// --------------------------------------------------------------------------------------

void CSoundManager::Update3D (int x, int y, int Nr)
{
	int   vol, pan;
	float xdiff, ydiff, Abstand;

	xdiff = ((pPlayer[0]->xpos + 45)  - x);
	ydiff = ((pPlayer[0]->ypos + 45)  - y);

	Abstand = float(sqrt((xdiff * xdiff) + (ydiff * ydiff)));

	vol = (int)((100-float(Abstand/6.0f)) / 100.0f * pSoundManager->its_GlobalSoundVolume);

	if (vol < 0)
		vol = 0;
	else

	// neuer Wert ist nicht leiser als der alte?

	{
		// Sound links oder rechts vom Spieler ?
		if (x < pPlayer[0]->xpos + 45)
		{
			pan = 128 - (100 - vol);
			if (pan < 0)
				pan = 0;
		}
		else
		{
			pan = 128 + (100 - vol);
			if (pan > 255)
				pan = 255;
		}


		SOUND_SetVolume (its_Sounds[Nr]->Channel, vol);
		SOUND_SetPan	 (its_Sounds[Nr]->Channel, 128);
	}
}

// --------------------------------------------------------------------------------------
// Wavedatei anhalten (falls sie geloopt wird zb)
// --------------------------------------------------------------------------------------

bool CSoundManager::StopWave(int Nr)
{
	if (its_Sounds[Nr] == NULL)
		return false;

	if (its_Sounds[Nr]->isPlaying == false)
		return false;

	SOUND_StopSound(its_Sounds[Nr]->Channel);

	its_Sounds[Nr]->isPlaying = false;
	its_Sounds[Nr]->Channel	 = -1;

	return true;
}

// --------------------------------------------------------------------------------------
// Alle Songs anhalten (Pause == true) oder wieder abspielen (Pause == false)
// --------------------------------------------------------------------------------------

void CSoundManager::PauseAllSongs(bool bPause)
{
	// alle pausieren
	//
	if (bPause == true)
	{
		// Musik stoppen
		//
		for (int i = 0; i < MAX_SONGS; i++)
		if (its_Songs[i] != NULL)
		{
			if (MUSIC_IsPlaying(its_Songs[i]->SongData))
				MUSIC_SetPaused(its_Songs[i]->SongData, true);
		}

		// Sounds stoppen
		//
		for (int i = 0; i < MAX_SOUNDS; i++)
		if (its_Sounds[i] != NULL)
			StopWave(i);
	}

	// alle wieder spielen lassen
	//
	else
	{
		// Musik weiter spielen
		//
		for (int i = 0; i < MAX_SONGS; i++)
		if (its_Songs[i] != NULL)
		{
				MUSIC_SetPaused(its_Songs[i]->SongData, false);
		}
	}
}

// --------------------------------------------------------------------------------------
// Alle Songs anhalten, die gerade spielen und diese merken
// --------------------------------------------------------------------------------------

void CSoundManager::PausePlaying(void)
{
	for (int i = 0; i < MAX_SONGS; i++)
	if (its_Songs[i] != NULL)
	{
		WasPlaying[i] = (MUSIC_IsPlaying(its_Songs[i]->SongData) &&
						 MUSIC_GetPaused(its_Songs[i]->SongData) == false);

		if (WasPlaying[i])
			MUSIC_SetPaused(its_Songs[i]->SongData, true);
	}
}

// --------------------------------------------------------------------------------------
// Alle Songs wieder spielen, die vorher angehalten wurden
// --------------------------------------------------------------------------------------

void CSoundManager::PlayPaused(void)
{
	for (int i = 0; i < MAX_SONGS; i++)
	if (its_Songs[i] != NULL)
	{
		if (WasPlaying[i])
			MUSIC_SetPaused(its_Songs[i]->SongData, false);
	}
}
